@@ -1,8 +1,8 @@
 #!/usr/bin/env texlua
 
--- Copyright 2016-2018 Brian Dunn
+-- Copyright 2016-2019 Brian Dunn
 
-printversion = "v0.67"
+printversion = "v0.72"
 requiredconfversion = "2" -- also at *lwarpmk.conf
 
 function printhelp ()
@@ -79,6 +79,20 @@ function cvalueerror ( line, linenum , cvalue )
     os.exit(1) ;
 end
 
+function printhowtorecompile ()
+-- Tells the user how to recompile to regenerate the configuration files.
+    print ("lwarpmk: The configuration files lwarpmk.conf and "..sourcename..".lwarpmkconf" )
+    print ("lwarpmk:   must be updated.  To do so, recompile" )
+    print ("lwarpmk:   " , sourcename..".tex" )
+    if ( printlatexcmd == "" ) then
+        print ("lwarpmk:   using xe/lua/pdflatex," )
+    else
+        print ("lwarpmk:   using the command:")
+        print ("lwarpmk:   " , printlatexcmd )
+    end
+    print ("lwarpmk:   then use lwarpmk again.")
+end -- printhowtorecompile
+
 function ignoreconf ()
 -- Global argument index
 argindex = 2
@@ -240,19 +254,14 @@ end --- for Windows
 if ( (package.config:sub(1,1)) ~= dirslash ) then
     print ("lwarpmk: ===")
     print ("lwarpmk: It appears that lwarpmk.conf is for a different operating system." )
-    print ("lwarpmk: To adjust lwarpmk.conf for the current operating system," )
-    print ("lwarpmk:   recompile the original document using xe/lua/pdflatex." )
-    print ("lwarpmk: ")
-    print ("lwarpmk: lwarpmk shall attempt to continue...")
+    printhowtorecompile ()
     print ("lwarpmk: ===")
+    os.exit(1)
 end
 -- Error if the configuration file's version is not current:
 if ( confversion ~= requiredconfversion ) then
     print ("lwarpmk: ===")
-    print ("lwarpmk: The configuration files lwarpmk.conf and "..sourcename..".lwarpmkconf" )
-    print ("lwarpmk:   must be updated.  To update the configuration files," )
-    print ("lwarpmk:   recompile "..sourcename..".tex using xe/lua/pdflatex," )
-    print ("lwarpmk:   then use lwarpmk again.")
+    printhowtorecompile ()
     print ("lwarpmk: ===")
     os.exit(1)
 end
@@ -388,7 +397,7 @@ function warnlimages ()
     print ("lwarpmk: \"" .. sourcename .. "-images.txt\" does not exist.")
     print ("lwarpmk: Your project does not use SVG math or other lateximages,")
     print ("lwarpmk: or the file has been deleted somehow.")
-    print ("lwarpmk: Use \"lwarpmk html\" to recompile your project")
+    print ("lwarpmk: Use \"lwarpmk html1\" to recompile your project")
     print ("lwarpmk: and recreate \"" .. sourcename .. "-images.txt\".")
     print ("lwarpmk: If your project does not use SVG math or other lateximages,")
     print ("lwarpmk: then \"" .. sourcename .. "-images.txt\" will never exist, and")
@@ -396,6 +405,16 @@ function warnlimages ()
     print ("lwarpmk: ===")
 end -- warnlimages
 
+function warnlimagesrecompile ()
+-- Warning if must recompile before creating limages:
+    print ("")
+    print ("lwarpmk: ===")
+    print ("lwarpmk: Cross-references are not yet correct.")
+    print ("lwarpmk: The document must be recompiled before creating the lateximages.")
+    print ("lwarpmk: Enter \"lwarpmk html1\" again, then try \"lwarpmk limages\" again.")
+    print ("lwarpmk: ===")
+end --warnlimagesrecompile
+
 function checklimages ()
 --
 -- Check <sourcename>.txt to see if need to recompile first.
@@ -424,12 +443,14 @@ for line in limagesfile:lines() do
         end
     end -- if i~=nil
 end -- do
+-- The last line should be |end|end|end|.
+-- If not, the compile must have aborted, and the images are incomplete.
+if ( lwimgpage ~= "end" ) then
+    warnlimagesrecompile()
+    os.exit(1) ;
+end
 if ( pagezerowarning ) then
-    print ("")
-    print ("lwarpmk: ===")
-    print ("lwarpmk: The document must be recompiled before creating the lateximages.")
-    print ("lwarpmk: Enter \"lwarpmk html\" again, then try \"lwarpmk limages\" again.")
-    print ("lwarpmk: ===")
+    warnlimagesrecompile()
     os.exit(1) ;
 end -- pagezerowarning
 end -- checklimages
@@ -509,6 +530,8 @@ if ( (i~=nil) ) then
     -- Skip if the page number is 0:
     if ( lwimgpage == "0" ) then
         pagezerowarning = true
+    -- Skip if the page number is "end":
+    else if ( lwimgpage == "end" ) then
     else
         -- Skip is this image is hashed and already exists:
         local lwimgfullname = imagesdirectory .. dirslash .. lwimgname .. ".svg"
@@ -531,6 +554,7 @@ if ( (i~=nil) ) then
                 createwindowsimage (lwimgfullname)
             end
         end -- not hashed or not exists
+    end -- not page "end"
     end -- not page 0
 end -- not nil
 end -- createonelateximage
@@ -541,7 +565,7 @@ function createlateximages ()
 --
 -- See if the document must be recompiled first:
 checklimages ()
--- See if the print version exists:
+-- See if the HTML version exists:
 checkhtmlpdfexists ()
 -- Attempt to create the lateximages:
 print ("lwarpmk: Creating lateximages.")
