@@ -14,7 +14,6 @@ local format, gsub, rep, sub, find = string.format, string.gsub, string.rep, str
 local load, dump = load, string.dump
 local tonumber, type, tostring, next, setmetatable = tonumber, type, tostring, next, setmetatable
 local unpack, concat = table.unpack, table.concat
-local unpack, concat = table.unpack, table.concat
 local P, V, C, S, R, Ct, Cs, Cp, Carg, Cc = lpeg.P, lpeg.V, lpeg.C, lpeg.S, lpeg.R, lpeg.Ct, lpeg.Cs, lpeg.Cp, lpeg.Carg, lpeg.Cc
 local patterns, lpegmatch = lpeg.patterns, lpeg.match
 local utfchar, utfbyte, utflen = utf.char, utf.byte, utf.len
@@ -22,25 +21,12 @@ local utfchar, utfbyte, utflen = utf.char, utf.byte, utf.len
 ----- loadstripped = utilities.lua.loadstripped
 ----- setmetatableindex = table.setmetatableindex
 
-local loadstripped = nil
-local oldfashioned = LUAVERSION < 5.2
-
-if oldfashioned then
-
-    loadstripped = function(str,shortcuts)
-        return load(str)
+local loadstripped = function(str,shortcuts)
+    if shortcuts then
+        return load(dump(load(str),true),nil,nil,shortcuts)
+    else
+        return load(dump(load(str),true))
     end
-
-else
-
-    loadstripped = function(str,shortcuts)
-        if shortcuts then
-            return load(dump(load(str),true),nil,nil,shortcuts)
-        else
-            return load(dump(load(str),true))
-        end
-    end
-
 end
 
 -- todo: make a special namespace for the formatter
@@ -598,66 +584,36 @@ local template = [[
 return function(%s) return %s end
 ]]
 
-local preamble, environment = "", { }
+local preamble = ""
 
-if oldfashioned then
+local environment = {
+    global          = global or _G,
+    lpeg            = lpeg,
+    type            = type,
+    tostring        = tostring,
+    tonumber        = tonumber,
+    format          = string.format,
+    concat          = table.concat,
+    signed          = number.signed,
+    points          = number.points,
+    basepoints      = number.basepoints,
+    utfchar         = utf.char,
+    utfbyte         = utf.byte,
+    lpegmatch       = lpeg.match,
+    nspaces         = string.nspaces,
+    utfpadding      = string.utfpadding,
+    tracedchar      = string.tracedchar,
+    autosingle      = string.autosingle,
+    autodouble      = string.autodouble,
+    sequenced       = table.sequenced,
+    formattednumber = number.formatted,
+    sparseexponent  = number.sparseexponent,
+    formattedfloat  = number.formattedfloat,
+    stripzero       = patterns.stripzero,
+    stripzeros      = patterns.stripzeros,
 
-    preamble = [[
-local lpeg=lpeg
-local type=type
-local tostring=tostring
-local tonumber=tonumber
-local format=string.format
-local concat=table.concat
-local signed=number.signed
-local points=number.points
-local basepoints= number.basepoints
-local utfchar=utf.char
-local utfbyte=utf.byte
-local lpegmatch=lpeg.match
-local nspaces=string.nspaces
-local utfpadding=string.utfpadding
-local tracedchar=string.tracedchar
-local autosingle=string.autosingle
-local autodouble=string.autodouble
-local sequenced=table.sequenced
-local formattednumber=number.formatted
-local sparseexponent=number.sparseexponent
-local formattedfloat=number.formattedfloat
-local stripzero=lpeg.patterns.stripzero
-local stripzeros=lpeg.patterns.stripzeros
-    ]]
-
-else
-
-    environment = {
-        global          = global or _G,
-        lpeg            = lpeg,
-        type            = type,
-        tostring        = tostring,
-        tonumber        = tonumber,
-        format          = string.format,
-        concat          = table.concat,
-        signed          = number.signed,
-        points          = number.points,
-        basepoints      = number.basepoints,
-        utfchar         = utf.char,
-        utfbyte         = utf.byte,
-        lpegmatch       = lpeg.match,
-        nspaces         = string.nspaces,
-        utfpadding      = string.utfpadding,
-        tracedchar      = string.tracedchar,
-        autosingle      = string.autosingle,
-        autodouble      = string.autodouble,
-        sequenced       = table.sequenced,
-        formattednumber = number.formatted,
-        sparseexponent  = number.sparseexponent,
-        formattedfloat  = number.formattedfloat,
-        stripzero       = lpeg.patterns.stripzero,
-        stripzeros      = lpeg.patterns.stripzeros,
-    }
-
-end
+    FORMAT          = string.f9,
+}
 
 -- -- --
 
@@ -775,9 +731,20 @@ local format_F = function(f) -- beware, no cast to number
     end
 end
 
+-- if string.f9 then
+--     format_F = function(f) -- beware, no cast to number
+--         n = n + 1
+--         if not f or f == "" then
+--             return format("(((a%s > -0.0000000005 and a%s < 0.0000000005) and '0') or FORMAT(a%s))",n,n,n,n,n)
+--         else
+--             return format("((a%s %% 1 == 0) and format('%%i',a%s) or FORMAT(a%s,'%%%sf'))",n,n,n,f)
+--         end
+--     end
+-- end
+
 local format_k = function(b,a) -- slow
     n = n + 1
-    return format("formattedfloat(a%s,%i,%i)",n,b or 0, a or 0)
+    return format("formattedfloat(a%s,%s,%s)",n,b or 0,a or 0)
 end
 
 local format_g = function(f)
@@ -1022,42 +989,100 @@ end
 
 --
 
+-- local strip
+--
+-- local format_Z = function(f)
+--     n = n + 1
+--     if not f or f == "" then
+--         f = ".9"
+--     end
+--     return format("(((a%s %% 1 == 0) and format('%%i',a%s)) or (strip and lpegmatch(stripzero,format('%%%sf',a%s))) or format('%%%sf',a%s))",n,n,f,n,f,n)
+-- end
+--
+-- function strings.stripformatterzeros()
+--     strip = true
+-- end
+
+-- add(formatters,"texexp", [[texexp(...)]], "local texexp = metapost.texexp")
+--
+-- add(formatters,"foo:bar",[[foo(...)]], { foo = function(...) print(...) return "!" end })
+-- print(string.formatters["foo %3!foo:bar! bar"](1,2,3))
+
+
 local format_rest = function(s)
     return format("%q",s) -- catches " and \n and such
 end
 
+-- local format_extension = function(extensions,f,name)
+--     local extension = extensions[name] or "tostring(%s)"
+--     local f = tonumber(f) or 1
+--     local w = find(extension,"%.%.%.")
+--     if f == 0 then
+--         if w then
+--             extension = gsub(extension,"%.%.%.","")
+--         end
+--         return extension
+--     elseif f == 1 then
+--         if w then
+--             extension = gsub(extension,"%.%.%.","%%s")
+--         end
+--         n = n + 1
+--         local a = "a" .. n
+--         return format(extension,a,a) -- maybe more times?
+--     elseif f < 0 then
+--         local a = "a" .. (n + f + 1)
+--         return format(extension,a,a)
+--     else
+--         if w then
+--             extension = gsub(extension,"%.%.%.",rep("%%s,",f-1).."%%s")
+--         end
+--         -- we could fill an array and then n = n + 1 unpack(t,n,n+f) but as we
+--         -- cache we don't save much and there are hardly any extensions anyway
+--         local t = { }
+--         for i=1,f do
+--             n = n + 1
+--          -- t[#t+1] = "a" .. n
+--             t[i] = "a" .. n
+--         end
+--         return format(extension,unpack(t))
+--     end
+-- end
+
 local format_extension = function(extensions,f,name)
     local extension = extensions[name] or "tostring(%s)"
     local f = tonumber(f) or 1
     local w = find(extension,"%.%.%.")
-    if f == 0 then
-        if w then
+    if w then
+        -- we have a wildcard
+        if f == 0 then
             extension = gsub(extension,"%.%.%.","")
-        end
-        return extension
-    elseif f == 1 then
-        if w then
+            return extension
+        elseif f == 1 then
             extension = gsub(extension,"%.%.%.","%%s")
-        end
-        n = n + 1
-        local a = "a" .. n
-        return format(extension,a,a) -- maybe more times?
-    elseif f < 0 then
-        local a = "a" .. (n + f + 1)
-        return format(extension,a,a)
-    else
-        if w then
-            extension = gsub(extension,"%.%.%.",rep("%%s,",f-1).."%%s")
-        end
-        -- we could fill an array and then n = n + 1 unpack(t,n,n+f) but as we
-        -- cache we don't save much and there are hardly any extensions anyway
-        local t = { }
-        for i=1,f do
             n = n + 1
-         -- t[#t+1] = "a" .. n
-            t[i] = "a" .. n
+            local a = "a" .. n
+            return format(extension,a,a) -- maybe more times?
+        elseif f < 0 then
+            local a = "a" .. (n + f + 1)
+            return format(extension,a,a)
+        else
+            extension = gsub(extension,"%.%.%.",rep("%%s,",f-1).."%%s")
+            -- we could fill an array and then n = n + 1 unpack(t,n,n+f) but as we
+            -- cache we don't save much and there are hardly any extensions anyway
+            local t = { }
+            for i=1,f do
+                n = n + 1
+             -- t[#t+1] = "a" .. n
+                t[i] = "a" .. n
+            end
+            return format(extension,unpack(t))
         end
-        return format(extension,unpack(t))
+    else
+        extension = gsub(extension,"%%s",function()
+            n = n + 1
+            return "a" .. n
+        end)
+        return extension
     end
 end
 
@@ -1155,6 +1180,7 @@ local builder = Cs { "start",
     ["M"] = (prefix_any * P("M")) / format_M, -- %M => xxx,xxx,xxx.xx (optional prefix instead of ,)
     --
     ["z"] = (prefix_any * P("z")) / format_z, -- %z => skip n arguments
+ -- ["Z"] = (prefix_any * P("Z")) / format_Z, -- %Z => optionally strip zeros
     --
     ["a"] = (prefix_any * P("a")) / format_a, -- %a => '...' (forces tostring)
     ["A"] = (prefix_any * P("A")) / format_A, -- %A => "..." (forces tostring)
@@ -1256,36 +1282,22 @@ strings.formatters = { }
 
 -- _connector_ is an experiment
 
-if oldfashioned then
-
-    function strings.formatters.new(noconcat)
-        local t = { _type_ = "formatter", _connector_ = noconcat and "," or "..", _extensions_ = { }, _preamble_ = preamble, _environment_ = { } }
-        setmetatable(t, { __index = make, __call = use })
-        return t
+function strings.formatters.new(noconcat)
+    local e = { } -- better make a copy as we can overload
+    for k, v in next, environment do
+        e[k] = v
     end
-
-else
-
-    function strings.formatters.new(noconcat)
-        local e = { } -- better make a copy as we can overload
-        for k, v in next, environment do
-            e[k] = v
-        end
-        local t = { _type_ = "formatter", _connector_ = noconcat and "," or "..", _extensions_ = { }, _preamble_ = "", _environment_ = e }
-        setmetatable(t, { __index = make, __call = use })
-        return t
-    end
-
+    local t = {
+        _type_        = "formatter",
+        _connector_   = noconcat and "," or "..",
+        _extensions_  = { },
+        _preamble_    = "",
+        _environment_ = e,
+    }
+    setmetatable(t, { __index = make, __call = use })
+    return t
 end
 
--- function strings.formatters.new()
---     local t = { _extensions_ = { }, _preamble_ = "", _type_ = "formatter", _n_ = 0 }
---     local m = { _t_ = t }
---     setmetatable(t, { __index = m, __call = use })
---     setmetatable(m, { __index = make })
---     return t
--- end
-
 local formatters   = strings.formatters.new() -- the default instance
 
 string.formatters  = formatters -- in the main string namespace
@@ -1316,19 +1328,9 @@ patterns.luaquoted = Cs(Cc('"') * ((1-S('"\n'))^1 + P('"')/'\\"' + P('\n')/'\\n"
 -- escaping by lpeg is faster for strings without quotes, slower on a string with quotes, but
 -- faster again when other q-escapables are found (the ones we don't need to escape)
 
-if oldfashioned then
-
-    add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],"local xmlescape = lpeg.patterns.xmlescape")
-    add(formatters,"tex",[[lpegmatch(texescape,%s)]],"local texescape = lpeg.patterns.texescape")
-    add(formatters,"lua",[[lpegmatch(luaescape,%s)]],"local luaescape = lpeg.patterns.luaescape")
-
-else
-
-    add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],{ xmlescape = lpeg.patterns.xmlescape })
-    add(formatters,"tex",[[lpegmatch(texescape,%s)]],{ texescape = lpeg.patterns.texescape })
-    add(formatters,"lua",[[lpegmatch(luaescape,%s)]],{ luaescape = lpeg.patterns.luaescape })
-
-end
+add(formatters,"xml",[[lpegmatch(xmlescape,%s)]],{ xmlescape = patterns.xmlescape })
+add(formatters,"tex",[[lpegmatch(texescape,%s)]],{ texescape = patterns.texescape })
+add(formatters,"lua",[[lpegmatch(luaescape,%s)]],{ luaescape = patterns.luaescape })
 
 -- -- yes or no:
 --
@@ -1398,3 +1400,39 @@ function strings.newcollector()
             end
         end
 end
+
+--
+
+local f_16_16 = formatters["%0.5N"]
+
+function number.to16dot16(n)
+    return f_16_16(n/65536.0)
+end
+
+--
+
+if not string.explode then
+
+    local tsplitat = lpeg.tsplitat
+
+    local p_utf   = patterns.utf8character
+    local p_check = C(p_utf) * (P("+") * Cc(true))^0
+    local p_split = Ct(C(p_utf)^0)
+    local p_space = Ct((C(1-P(" ")^1) + P(" ")^1)^0)
+
+    function string.explode(str,symbol)
+        if symbol == "" then
+            return lpegmatch(p_split,str)
+        elseif symbol then
+            local a, b = lpegmatch(p_check,symbol)
+            if b then
+                return lpegmatch(tsplitat(P(a)^1),str)
+            else
+                return lpegmatch(tsplitat(a),str)
+            end
+        else
+            return lpegmatch(p_space,str)
+        end
+    end
+
+end
