@@ -49,6 +49,8 @@ constructors.cache           = containers.define("fonts", "constructors", constr
 constructors.privateoffset   = fonts.privateoffsets.textbase or 0xF0000
 constructors.cacheintex      = true -- so we see the original table in fonts.font
 
+constructors.addtounicode    = true
+
 -- This might become an interface:
 
 local designsizes           = allocate()
@@ -424,7 +426,10 @@ function constructors.scale(tfmdata,specification)
     targetparameters.forcedsize  = forcedsize  -- context specific
     targetparameters.extrafactor = extrafactor -- context specific
     --
+    local addtounicode  = constructors.addtounicode
+    --
     local tounicode     = fonts.mappings.tounicode
+    local unknowncode   = tounicode(0xFFFD)
     --
     local defaultwidth  = resources.defaultwidth  or 0
     local defaultheight = resources.defaultheight or 0
@@ -542,7 +547,6 @@ function constructors.scale(tfmdata,specification)
     local hasitalics       = properties.hasitalics
     local autoitalicamount = properties.autoitalicamount
     local stackmath        = not properties.nostackmath
-    local nonames          = properties.noglyphnames
     local haskerns         = properties.haskerns     or properties.mode == "base" -- we can have afm in node mode
     local hasligatures     = properties.hasligatures or properties.mode == "base" -- we can have afm in node mode
     local realdimensions   = properties.realdimensions
@@ -671,9 +675,10 @@ function constructors.scale(tfmdata,specification)
             description = descriptions[unicode] or character
             index       = description.index or unicode
         end
-        local width  = description.width
-        local height = description.height
-        local depth  = description.depth
+        local width     = description.width
+        local height    = description.height
+        local depth     = description.depth
+        local isunicode = description.unicode
         if realdimensions then
             -- this is mostly for checking issues
             if not height or height == 0 then
@@ -700,16 +705,16 @@ function constructors.scale(tfmdata,specification)
     --  if depth  then depth  = vdelta*depth  else depth  = scaleddepth  end
         if depth and depth ~= 0 then
             depth = delta*depth
-            if nonames then
+            if isunicode then
                 chr = {
-                    index  = index,
-                    height = height,
-                    depth  = depth,
-                    width  = width,
+                    index   = index,
+                    height  = height,
+                    depth   = depth,
+                    width   = width,
+                    unicode = isunicode,
                 }
             else
                 chr = {
-                    name   = description.name,
                     index  = index,
                     height = height,
                     depth  = depth,
@@ -717,28 +722,23 @@ function constructors.scale(tfmdata,specification)
                 }
             end
         else
-            -- this saves a little bit of memory time and memory, esp for big cjk fonts
-            if nonames then
+            if isunicode then
                 chr = {
-                    index  = index,
-                    height = height,
-                    width  = width,
+                    index   = index,
+                    height  = height,
+                    width   = width,
+                    unicode = isunicode,
                 }
             else
                 chr = {
-                    name   = description.name,
                     index  = index,
                     height = height,
                     width  = width,
                 }
             end
         end
-        local isunicode = description.unicode
-        if isunicode then
-            chr.unicode   = isunicode
-            chr.tounicode = tounicode(isunicode)
-            -- in luatex > 0.85 we can do this:
-            -- chr.tounicode = isunicode
+        if addtounicode then
+            chr.tounicode = isunicode and tounicode(isunicode) or unknowncode
         end
         if hasquality then
             -- we could move these calculations elsewhere (saves calculations)
@@ -813,7 +813,10 @@ function constructors.scale(tfmdata,specification)
             if stackmath then
                 local mk = character.mathkerns
                 if mk then
-                    local tr, tl, br, bl = mk.topright, mk.topleft, mk.bottomright, mk.bottomleft
+                    local tr = mk.topright
+                    local tl = mk.topleft
+                    local br = mk.bottomright
+                    local bl = mk.bottomleft
                     chr.mathkern = { -- singular -> should be patched in luatex !
                         top_right    = tr and mathkerns(tr,vdelta) or nil,
                         top_left     = tl and mathkerns(tl,vdelta) or nil,
@@ -923,10 +926,9 @@ function constructors.scale(tfmdata,specification)
     -- catch inconsistencies
     --
     local vfonts = target.fonts
---     if isvirtual then
-if isvirtual or target.type == "virtual" or properties.virtualized then
+    if isvirtual or target.type == "virtual" or properties.virtualized then
         properties.virtualized = true
-target.type = "virtual"
+        target.type = "virtual"
         if not vfonts or #vfonts == 0 then
             target.fonts = { { id = 0 } }
         end
@@ -1024,24 +1026,22 @@ function constructors.finalize(tfmdata)
         properties.virtualized = tfmdata.type == "virtual"
     end
     --
-    if not tfmdata.properties then
-        tfmdata.properties = {
-            fontname      = tfmdata.fontname,
-            filename      = tfmdata.filename,
-            fullname      = tfmdata.fullname,
-            name          = tfmdata.name,
-            psname        = tfmdata.psname,
-            --
-            encodingbytes = tfmdata.encodingbytes or 1,
-            embedding     = tfmdata.embedding     or "subset",
-            tounicode     = tfmdata.tounicode     or 1,
-            cidinfo       = tfmdata.cidinfo       or nil,
-            format        = tfmdata.format        or "type1",
-            direction     = tfmdata.direction     or 0,
-            writingmode   = tfmdata.writingmode   or "horizontal",
-            identity      = tfmdata.identity      or "horizontal",
-        }
-    end
+    properties.fontname      = properties.fontname or tfmdata.fontname
+    properties.filename      = properties.filename or tfmdata.filename
+    properties.fullname      = properties.fullname or tfmdata.fullname
+    properties.name          = properties.name     or tfmdata.name
+    properties.psname        = properties.psname   or tfmdata.psname
+    --
+    properties.encodingbytes = tfmdata.encodingbytes or 1
+    properties.embedding     = tfmdata.embedding     or "subset"
+    properties.tounicode     = tfmdata.tounicode     or 1
+    properties.cidinfo       = tfmdata.cidinfo       or nil
+    properties.format        = tfmdata.format        or "type1"
+    properties.direction     = tfmdata.direction     or 0
+    properties.writingmode   = tfmdata.writingmode   or "horizontal"
+    properties.identity      = tfmdata.identity      or "horizontal"
+    properties.usedbitmap    = tfmdata.usedbitmap
+    --
     if not tfmdata.resources then
         tfmdata.resources = { }
     end
@@ -1163,7 +1163,9 @@ loose our testcases for <l n='luatex'/>.</p>
 --ldx]]--
 
 function constructors.hashinstance(specification,force)
-    local hash, size, fallbacks = specification.hash, specification.size, specification.fallbacks
+    local hash      = specification.hash
+    local size      = specification.size
+    local fallbacks = specification.fallbacks
     if force or not hash then
         hash = constructors.hashfeatures(specification)
         specification.hash = hash
@@ -1601,7 +1603,8 @@ end
 -- while typesetting
 
 function constructors.collectprocessors(what,tfmdata,features,trace,report)
-    local processes, nofprocesses = { }, 0
+    local processes    = { }
+    local nofprocesses = 0
     if features and next(features) then
         local properties     = tfmdata.properties
         local whathandler    = handlers[what]
